//================================================================
// Przykladowa petla glowna implementujaca symulacje stalokrokowa.
//
//
// Przyklad dolaczony artykulu "Jak nalezy pisac aktualizacje stanu gry"
// (C) 2010, Jacek "TeMPOraL" Zlydach
// http://temporal.pr0.pl/devblog/
//
// Kod moze byc uzywany w dowolnych celach.
//================================================================

// Funkcje zewnetrzne
float GetCurrentTime();	//zwraca aktualny czas systemowy w sekundach
void GrabInput();	//uzywana do zebrania danych z klawiatury, myszki, ect.
void UpdateGame(float dT);	//aktualizuje stan gry
void Render();	//odrysowuje ekran

// Glowna czesc przykladu
template <typename T>
T clamp(const T& what, const T& a, const T& b)
{
	return std::min(b, std::max(what, a));
}


int main()
{
	float dt = 0.0f;	//czas od ostatniej aktualizacji
	float lastUpdateTime = GetCurrentTime();	//czas ostatniej aktualizacji
												//przykladowa funkcja GetCurrentTime() pobiera
												//nam od systemu aktualny czas w sekundach
	float accumulator = 0.0f;
	const float TIME_STEP = 0.03;	//krok czasowy, a zarazem czas trwania ramki
									//fizyki w sekundach; tutaj 30 milisekund, czyli
									//ok. 30 aktualizacji na sekund
	const float MAX_ACCUMULATED_TIME = 1.0;	//maksymalny czas zgromadzony w pojedynczym
											//obiegu petli glownej
	while(true)
	{
		dt = GetCurrentTime() - lastUpdateTime;	//obliczenie czasu od ostatniej klatki
		lastUpdate += dt;	//podmiana
		dt = std::max(0, dt);	//upewniamy sie, ze dt >= 0
		accumulator += dt;
		accumulator = clamp(accumulator, 0, MAX_ACCUMULATED_TIME);	//zapobiegamy
																	//zbyt duzej ilosci aktualizacji w danym obiegu
																	//petli glownej
		GrabInput();	//<-- zbieranie wejscia z klawiatury, myszki, sieci, itp.
		while(accumulator > TIME_STEP)
		{
			UpdateGame(TIME_STEP);	//<-- aktualizacja fizyki i logiki gry
			accumulator -= TIME_STEP;
		}
		RenderGame();	//<-- wyswietlenie aktualnego stanu na ekranie
	}
}
